#![warn(clippy::all)]

mod cli;
mod client;
mod config;
mod gitlab;
mod jira;
mod observe;
mod prelude;
mod result;
mod task;
mod tasks;
mod tui;

use config::Config;

use crate::cli::{Cli, Commands};
use crate::prelude::*;

#[tokio::main]
async fn main() -> Result<()> {
    let cli = Cli::new();
    let conf = Config::load(cli.config_file_path)?;

    // this guard causes logs to be flushed when dropped (which is at the end of
    // this function which would be the end of our program)
    // https://docs.rs/tracing-appender/latest/tracing_appender/non_blocking/struct.WorkerGuard.html
    let _log_guard = observe::setup_logging(cli.logs_directory, cli.tracing_env_filter)?;

    let tasks = Arc::new(crate::tasks::Tasks::try_new(conf, cli.data_directory)?);

    let result = match cli.command {
        Commands::Ui(args) => args.run(tasks).await,
        Commands::Purge(args) => args.run(tasks).await,
        Commands::Cleanup(args) => args.run(tasks).await,
        Commands::Sync(args) => args.sync(tasks).await,
        Commands::List(args) => args.list(tasks).await,
        Commands::Jira(jira) => jira.exec(tasks).await,
        Commands::Gitlab(gitlab) => gitlab.exec(tasks).await,
    };
    result
}
